# -*- coding: utf-8 -*-
from distutils.core import setup

setup(
    name='cfget',
    version='0.19',
    description='get information out of config files',
    author='Enrico Zini',
    author_email='enrico@enricozini.org',
    #url='',
    scripts = ["cfget"],
)
